@@ -312,6 +312,16 @@ impl From<SegmentError> for IndexError {
     }
 }
 
+impl From<ReadError> for IndexError {
+    fn from(e: ReadError) -> Self {
+        match e {
+            ReadError::Generic(ex) => IndexError::Generic(ex),
+            ReadError::SegmentNotFound => IndexError::IndexNotFound,
+            ReadError::InvalidPersyId(_) => unreachable!(),
+        }
+    }
+}
+
 #[derive(Debug, Error)]
 pub enum GenericError {
     #[error("IO Error: {from}")]
@@ -808,3 +818,16 @@ impl From<io::Error> for CreateError {
         }
     }
 }
+
+impl From<OpenError> for CreateError {
+    fn from(open: OpenError) -> Self {
+        match open {
+            OpenError::Generic(g) => g.into(),
+            OpenError::NotExists => unreachable!(),
+            OpenError::InitError(_) => unreachable!(),
+            OpenError::AlreadyInUse(x) => CreateError::AlreadyInUse(x),
+            OpenError::NotPersyFile => unreachable!(),
+            OpenError::AlreadyExists => CreateError::AlreadyExists,
+        }
+    }
+}
