@@ -1,6 +1,8 @@
+pub mod bytevec;
 pub mod config;
 pub mod iter;
 pub mod keeper;
 pub mod serialization;
+pub(crate) mod string_wrapper;
 pub mod tree;
 pub mod value_iter;
