@@ -15,7 +15,7 @@ use crate::{
 };
 use std::{
     collections::hash_map::HashMap,
-    sync::{Arc, Mutex, MutexGuard},
+    sync::{Arc, Mutex},
 };
 pub(crate) mod records;
 
@@ -96,6 +96,229 @@ struct JournalShared {
     to_free: Option<JournalPagesToFree>,
 }
 
+impl JournalShared {
+    fn new_version_0(mut page: ReadPage, all: &Allocator) -> PERes<JournalShared> {
+        let first_page;
+        let current;
+        {
+            let buffer = all.read_root_journal(&mut page, 11);
+            first_page = read_u64(&buffer[0..8]);
+        }
+        current = if first_page != 0 {
+            all.write_page(first_page)?
+        } else {
+            // Empty 0 sized page
+            Page::new(Vec::new(), 0, 0, 0)
+        };
+        Ok(JournalShared {
+            root: page.get_index(),
+            first_page,
+            last_page: first_page,
+            last_pos: 0,
+            starts: StartList::new(),
+            current,
+            to_clear: Vec::new(),
+            to_free: None,
+        })
+    }
+
+    fn required_space(&mut self, space: u32, allocator: &Allocator) -> PERes<()> {
+        // if there is no page or the  'current content' + 'space required' + 'end marker' is more
+        // than the page, allocate new page and link the previous one
+        if self.last_pos + space + 1 >= JOURNAL_PAGE_SIZE as u32 {
+            let prev = self.last_page;
+            let last_pos = self.last_pos;
+            let new_page = allocator.allocate(JOURNAL_PAGE_EXP)?;
+            let new_index = new_page.get_index();
+            let mut old = std::mem::replace(&mut self.current, new_page);
+            if prev != 0 {
+                old.seek(JOURNAL_PAGE_NEXT_OFFSET);
+                old.write_u64(new_index);
+                old.seek(last_pos);
+                old.write_u8(0);
+                allocator.flush_page(old)?;
+            }
+            self.last_page = new_index;
+            self.current.seek(JOURNAL_PAGE_PREV_OFFSET);
+            self.current.write_u64(prev);
+            allocator.flush_journal(&self.current)?;
+            self.last_pos = JOURNAL_PAGE_CONTENT_OFFSET;
+        }
+        Ok(())
+    }
+
+    fn append_buffer(&mut self, buffer: &[u8]) -> PERes<(u64, u32)> {
+        let cur_page = self.last_page;
+        let cur_pos = self.last_pos;
+        self.current.seek(cur_pos);
+        self.current.write_all(&*buffer);
+        self.last_pos += buffer.len() as u32;
+        Ok((cur_page, cur_pos))
+    }
+
+    fn log_bytes(&mut self, buffer: &[u8], allocator: &Allocator, flush: bool) -> PERes<()> {
+        self.required_space(buffer.len() as u32, allocator)?;
+        self.append_buffer(&buffer)?;
+        if flush {
+            allocator.flush_journal(&self.current)?;
+        }
+        Ok(())
+    }
+
+    pub fn cleaned_to_trim(&mut self, ids: &[JournalId]) -> PERes<()> {
+        self.to_clear.extend_from_slice(ids);
+        Ok(())
+    }
+
+    pub fn start(&mut self, allocator: &Allocator) -> PERes<JournalId> {
+        let buffer = Journal::prepare_buffer(&Start::default(), &JournalId::new(0, 0))?;
+        self.required_space(buffer.len() as u32, &allocator)?;
+        let val = self.append_buffer(&buffer)?;
+        let id = JournalId::new(val.0, val.1);
+        self.starts.push(&id);
+        Ok(id)
+    }
+
+    pub fn clear_in_queue(&mut self, allocator: &Allocator) -> PERes<()> {
+        let mut pages_to_free = Vec::new();
+        let ids = self.to_clear.clone();
+        self.to_clear.clear();
+        let mut new_first = None;
+        for id in ids {
+            if self.starts.remove(&id) {
+                let first_page = self.first_page;
+                let mut free_cursor = id.page;
+                loop {
+                    let read = if free_cursor == self.last_page {
+                        let pos = self.current.cursor_pos();
+                        self.current.seek(JOURNAL_PAGE_PREV_OFFSET);
+                        let prev = self.current.read_u64();
+                        self.current.seek(pos as u32);
+                        prev
+                    } else {
+                        let mut cur = allocator.load_page(free_cursor)?;
+                        cur.seek(JOURNAL_PAGE_PREV_OFFSET);
+                        cur.read_u64()
+                    };
+                    if free_cursor != id.page {
+                        pages_to_free.push(free_cursor)
+                    }
+                    if free_cursor == first_page {
+                        break;
+                    }
+                    free_cursor = read;
+                }
+                new_first = Some(id.page);
+                self.first_page = id.page;
+            }
+        }
+        if let Some(first) = new_first {
+            let mut buffer = [0; 11];
+            write_u64(&mut buffer[0..8], first);
+            let root = allocator.write_page(self.root)?;
+            allocator.write_journal_root(root, &mut buffer, JOURNAL_ROOT_VERSION)?;
+        }
+        self.free_pages_tx(pages_to_free, allocator)?;
+        Ok(())
+    }
+
+    pub fn free_pages_tx(&mut self, pages_to_free: Vec<u64>, allocator: &Allocator) -> PERes<()> {
+        //tx
+        let to_free_list = self.to_free.take();
+        if let Some(to_free) = to_free_list {
+            for page in to_free.pages {
+                allocator.free(page)?;
+            }
+            // Enqueue for clear the free pages journal pages tx
+            self.finished_to_clean(&[to_free.free_tx_id], allocator)?;
+            allocator.flush_free_list()?;
+        }
+        if !pages_to_free.is_empty() {
+            // Log the pages removed from the journal for free
+            let id = self.start(allocator)?;
+            for page in &pages_to_free {
+                self.log_bytes(&Journal::prepare_buffer(&FreedPage::new(*page), &id)?, allocator, false)?;
+            }
+            self.log_bytes(&Journal::prepare_buffer(&PrepareCommit::new(), &id)?, allocator, false)?;
+            self.log_bytes(&Journal::prepare_buffer(&Commit::new(), &id)?, allocator, true)?;
+            self.to_free = Some(JournalPagesToFree {
+                free_tx_id: id,
+                pages: pages_to_free,
+            });
+        }
+        Ok(())
+    }
+
+    pub fn finished_to_clean(&mut self, ids: &[JournalId], allocator: &Allocator) -> PERes<()> {
+        let mut iter = ids.iter().peekable();
+        while let Some(id) = iter.next() {
+            self.log_bytes(
+                &Journal::prepare_buffer(&Cleanup::new(), id)?,
+                allocator,
+                iter.peek().is_none(),
+            )?;
+        }
+        self.cleaned_to_trim(ids)?;
+        Ok(())
+    }
+
+    fn recover<T>(&mut self, mut found: T, allocator: &Allocator) -> PERes<()>
+    where
+        T: FnMut(&dyn JournalEntry, &JournalId),
+    {
+        let mut cur_page = self.first_page;
+        self.last_page = self.first_page;
+        let mut page = allocator.load_page(cur_page)?;
+        page.seek(JOURNAL_PAGE_CONTENT_OFFSET);
+        loop {
+            let cursor_pos = page.cursor_pos();
+            let tp = page.read_u8();
+            if tp == 0 {
+                let last_pos = page.cursor_pos() as u32;
+                page.seek(JOURNAL_PAGE_NEXT_OFFSET);
+                cur_page = page.read_u64();
+                if cur_page == 0 {
+                    self.last_pos = last_pos - 1;
+                    break;
+                }
+                page = allocator.load_page(cur_page)?;
+                page.seek(JOURNAL_PAGE_CONTENT_OFFSET);
+                self.last_page = cur_page;
+            } else {
+                let page_id = page.read_varint_u64();
+                let pos = page.read_varint_u32();
+                let id = JournalId::new(page_id, pos);
+                let ref_page = &mut page;
+                let ref_found = &mut found;
+                match tp {
+                    //The Start entry has no valid id, should not be recovered
+                    1 => {
+                        Start::default().read(&mut page)?;
+                        let tx_id = JournalId::new(self.last_page, cursor_pos as u32);
+                        self.starts.push(&tx_id);
+                    }
+                    2 => recover_entry(&mut InsertRecord::default(), ref_page, ref_found, &id)?,
+                    3 => recover_entry(&mut PrepareCommit::default(), ref_page, ref_found, &id)?,
+                    4 => recover_entry(&mut Commit::default(), ref_page, ref_found, &id)?,
+                    5 => recover_entry(&mut UpdateRecord::default(), ref_page, ref_found, &id)?,
+                    6 => recover_entry(&mut DeleteRecord::default(), ref_page, ref_found, &id)?,
+                    7 => recover_entry(&mut Rollback::default(), ref_page, ref_found, &id)?,
+                    8 => recover_entry(&mut CreateSegment::default(), ref_page, ref_found, &id)?,
+                    9 => recover_entry(&mut DropSegment::default(), ref_page, ref_found, &id)?,
+                    10 => recover_entry(&mut ReadRecord::default(), ref_page, ref_found, &id)?,
+                    11 => recover_entry(&mut Metadata::default(), ref_page, ref_found, &id)?,
+                    12 => recover_entry(&mut FreedPage::default(), ref_page, ref_found, &id)?,
+                    13 => recover_entry(&mut NewSegmentPage::default(), ref_page, ref_found, &id)?,
+                    14 => recover_entry(&mut Cleanup::default(), ref_page, ref_found, &id)?,
+                    _ => panic!(" wrong log entry {} ", tp),
+                };
+            }
+        }
+        self.current = allocator.write_page(self.last_page)?;
+        Ok(())
+    }
+}
+
 /// Journal segment is the area where the transactional log is kept
 pub struct Journal {
     allocator: Arc<Allocator>,
@@ -128,7 +351,7 @@ impl Journal {
     pub fn new(all: &Arc<Allocator>, page: u64) -> PERes<Journal> {
         let mut page = all.load_page(page)?;
         let journal = match page.read_u8() {
-            JOURNAL_ROOT_VERSION_0 => Self::new_version_0(page, all)?,
+            JOURNAL_ROOT_VERSION_0 => JournalShared::new_version_0(page, all)?,
             _ => panic!("version not supported"),
         };
         Ok(Journal {
@@ -136,30 +359,6 @@ impl Journal {
             journal: Mutex::new(journal),
         })
     }
-    fn new_version_0(mut page: ReadPage, all: &Allocator) -> PERes<JournalShared> {
-        let first_page;
-        let current;
-        {
-            let buffer = all.read_root_journal(&mut page, 11);
-            first_page = read_u64(&buffer[0..8]);
-        }
-        current = if first_page != 0 {
-            all.write_page(first_page)?
-        } else {
-            // Empty 0 sized page
-            Page::new(Vec::new(), 0, 0, 0)
-        };
-        Ok(JournalShared {
-            root: page.get_index(),
-            first_page,
-            last_page: first_page,
-            last_pos: 0,
-            starts: StartList::new(),
-            current,
-            to_clear: Vec::new(),
-            to_free: None,
-        })
-    }
 
     pub fn init(allocator: &Allocator) -> PERes<u64> {
         let root_page = allocator.allocate(5)?;
@@ -172,13 +371,7 @@ impl Journal {
     }
 
     pub fn start(&self) -> PERes<JournalId> {
-        let buffer = Self::prepare_buffer(&Start::default(), &JournalId::new(0, 0))?;
-        let mut jr = self.journal.lock()?;
-        self.required_space(buffer.len() as u32, &mut jr)?;
-        let val = Self::append_buffer(&mut jr, &buffer)?;
-        let id = JournalId::new(val.0, val.1);
-        jr.starts.push(&id);
-        Ok(id)
+        self.journal.lock()?.start(&self.allocator)
     }
 
     pub(crate) fn prepare(&self, entry: &dyn JournalEntry, id: &JournalId) -> PERes<()> {
@@ -192,99 +385,17 @@ impl Journal {
     }
 
     pub fn clear_in_queue(&self) -> PERes<()> {
-        let mut pages_to_free = Vec::new();
-        {
-            let mut lock = self.journal.lock()?;
-            let ids = lock.to_clear.clone();
-            lock.to_clear.clear();
-            let mut new_first = None;
-            for id in ids {
-                if lock.starts.remove(&id) {
-                    let first_page = lock.first_page;
-                    let mut free_cursor = id.page;
-                    loop {
-                        let read = if free_cursor == lock.last_page {
-                            let pos = lock.current.cursor_pos();
-                            lock.current.seek(JOURNAL_PAGE_PREV_OFFSET);
-                            let prev = lock.current.read_u64();
-                            lock.current.seek(pos as u32);
-                            prev
-                        } else {
-                            let mut cur = self.allocator.load_page(free_cursor)?;
-                            cur.seek(JOURNAL_PAGE_PREV_OFFSET);
-                            cur.read_u64()
-                        };
-                        if free_cursor != id.page {
-                            pages_to_free.push(free_cursor)
-                        }
-                        if free_cursor == first_page {
-                            break;
-                        }
-                        free_cursor = read;
-                    }
-                    new_first = Some(id.page);
-                    lock.first_page = id.page;
-                }
-            }
-            if let Some(first) = new_first {
-                let mut buffer = [0; 11];
-                write_u64(&mut buffer[0..8], first);
-                let root = self.allocator.write_page(lock.root)?;
-                self.allocator
-                    .write_journal_root(root, &mut buffer, JOURNAL_ROOT_VERSION)?;
-            }
-        }
-        self.free_pages_tx(pages_to_free)?;
-        Ok(())
-    }
-    pub fn free_pages_tx(&self, pages_to_free: Vec<u64>) -> PERes<()> {
-        let mut prev_tx_id = None;
-        {
-            //Free the pages of previous clear, logged 2 tx ahead, now synced because of the second
-            //tx
-            let to_free_list = { self.journal.lock()?.to_free.take() };
-            if let Some(to_free) = to_free_list {
-                for page in to_free.pages {
-                    self.allocator.free(page)?;
-                }
-                prev_tx_id = Some(to_free.free_tx_id);
-            }
-        }
-        if let Some(id) = prev_tx_id {
-            // Enqueue for clear the free pages journal pages tx
-            self.finished_to_clean(&[id])?;
-        }
-        if !pages_to_free.is_empty() {
-            // Log the pages removed from the journal for free
-            let id = self.start()?;
-            for page in &pages_to_free {
-                self.log(&FreedPage::new(*page), &id)?;
-            }
-            self.log(&PrepareCommit::new(), &id)?;
-            self.end(&Commit::new(), &id)?;
-            let mut lock = self.journal.lock()?;
-            lock.to_free = Some(JournalPagesToFree {
-                free_tx_id: id,
-                pages: pages_to_free,
-            });
-        }
-        Ok(())
+        self.journal.lock()?.clear_in_queue(&self.allocator)
     }
 
     pub fn finished_to_clean(&self, ids: &[JournalId]) -> PERes<()> {
-        let mut iter = ids.iter().peekable();
-        while let Some(id) = iter.next() {
-            self.internal_log(&Cleanup::new(), id, iter.peek().is_none())?;
-        }
         let mut lock = self.journal.lock()?;
-        lock.to_clear.extend_from_slice(ids);
-        Ok(())
+        lock.finished_to_clean(ids, &self.allocator)
     }
 
     pub fn cleaned_to_trim(&self, ids: &[JournalId]) -> PERes<()> {
         let mut lock = self.journal.lock()?;
-        lock.to_clear.extend_from_slice(ids);
-        Ok(())
+        lock.cleaned_to_trim(ids)
     }
 
     pub(crate) fn log(&self, entry: &dyn JournalEntry, id: &JournalId) -> PERes<()> {
@@ -301,105 +412,18 @@ impl Journal {
         Ok(buffer)
     }
 
-    fn append_buffer(jr: &mut MutexGuard<JournalShared>, buffer: &[u8]) -> PERes<(u64, u32)> {
-        let cur_page = jr.last_page;
-        let cur_pos = jr.last_pos;
-        jr.current.seek(cur_pos);
-        jr.current.write_all(&*buffer);
-        jr.last_pos += buffer.len() as u32;
-        Ok((cur_page, cur_pos))
-    }
     fn internal_log(&self, entry: &dyn JournalEntry, id: &JournalId, flush: bool) -> PERes<()> {
         let buffer = Self::prepare_buffer(entry, id)?;
         let mut jr = self.journal.lock()?;
-        self.required_space(buffer.len() as u32, &mut jr)?;
-        Self::append_buffer(&mut jr, &buffer)?;
-        if flush {
-            self.allocator.flush_journal(&jr.current)?;
-        }
-        Ok(())
+        jr.log_bytes(&buffer, &self.allocator, flush)
     }
 
-    pub(crate) fn recover<T>(&self, mut found: T) -> PERes<()>
+    pub(crate) fn recover<T>(&self, found: T) -> PERes<()>
     where
         T: FnMut(&dyn JournalEntry, &JournalId),
     {
         let mut jr = self.journal.lock()?;
-        let mut cur_page = jr.first_page;
-        jr.last_page = jr.first_page;
-        let mut page = self.allocator.load_page(cur_page)?;
-        page.seek(JOURNAL_PAGE_CONTENT_OFFSET);
-        loop {
-            let cursor_pos = page.cursor_pos();
-            let tp = page.read_u8();
-            if tp == 0 {
-                let last_pos = page.cursor_pos() as u32;
-                page.seek(JOURNAL_PAGE_NEXT_OFFSET);
-                cur_page = page.read_u64();
-                if cur_page == 0 {
-                    jr.last_pos = last_pos - 1;
-                    break;
-                }
-                page = self.allocator.load_page(cur_page)?;
-                page.seek(JOURNAL_PAGE_CONTENT_OFFSET);
-                jr.last_page = cur_page;
-            } else {
-                let page_id = page.read_varint_u64();
-                let pos = page.read_varint_u32();
-                let id = JournalId::new(page_id, pos);
-                let ref_page = &mut page;
-                let ref_found = &mut found;
-                match tp {
-                    //The Start entry has no valid id, should not be recovered
-                    1 => {
-                        Start::default().read(&mut page)?;
-                        let tx_id = JournalId::new(jr.last_page, cursor_pos as u32);
-                        jr.starts.push(&tx_id);
-                    }
-                    2 => recover_entry(&mut InsertRecord::default(), ref_page, ref_found, &id)?,
-                    3 => recover_entry(&mut PrepareCommit::default(), ref_page, ref_found, &id)?,
-                    4 => recover_entry(&mut Commit::default(), ref_page, ref_found, &id)?,
-                    5 => recover_entry(&mut UpdateRecord::default(), ref_page, ref_found, &id)?,
-                    6 => recover_entry(&mut DeleteRecord::default(), ref_page, ref_found, &id)?,
-                    7 => recover_entry(&mut Rollback::default(), ref_page, ref_found, &id)?,
-                    8 => recover_entry(&mut CreateSegment::default(), ref_page, ref_found, &id)?,
-                    9 => recover_entry(&mut DropSegment::default(), ref_page, ref_found, &id)?,
-                    10 => recover_entry(&mut ReadRecord::default(), ref_page, ref_found, &id)?,
-                    11 => recover_entry(&mut Metadata::default(), ref_page, ref_found, &id)?,
-                    12 => recover_entry(&mut FreedPage::default(), ref_page, ref_found, &id)?,
-                    13 => recover_entry(&mut NewSegmentPage::default(), ref_page, ref_found, &id)?,
-                    14 => recover_entry(&mut Cleanup::default(), ref_page, ref_found, &id)?,
-                    _ => panic!(" wrong log entry {} ", tp),
-                };
-            }
-        }
-        jr.current = self.allocator.write_page(jr.last_page)?;
-        Ok(())
-    }
-
-    fn required_space(&self, space: u32, jr: &mut MutexGuard<JournalShared>) -> PERes<()> {
-        // if there is no page or the  'current content' + 'space required' + 'end marker' is more
-        // than the page, allocate new page and link the previous one
-        if jr.last_pos + space + 1 >= JOURNAL_PAGE_SIZE as u32 {
-            let prev = jr.last_page;
-            let last_pos = jr.last_pos;
-            let new_page = self.allocator.allocate(JOURNAL_PAGE_EXP)?;
-            let new_index = new_page.get_index();
-            let mut old = std::mem::replace(&mut jr.current, new_page);
-            if prev != 0 {
-                old.seek(JOURNAL_PAGE_NEXT_OFFSET);
-                old.write_u64(new_index);
-                old.seek(last_pos);
-                old.write_u8(0);
-                self.allocator.flush_page(old)?;
-            }
-            jr.last_page = new_index;
-            jr.current.seek(JOURNAL_PAGE_PREV_OFFSET);
-            jr.current.write_u64(prev);
-            self.allocator.flush_journal(&jr.current)?;
-            jr.last_pos = JOURNAL_PAGE_CONTENT_OFFSET;
-        }
-        Ok(())
+        jr.recover(found, &self.allocator)
     }
 }
 
