@@ -146,9 +146,6 @@ impl Segment {
         }
         Ok(pages)
     }
-    pub fn set_last_page(&mut self, page: u64) {
-        self.last_page = page;
-    }
 }
 
 pub(crate) trait SegmentPageRead: PageOps {
@@ -410,12 +407,6 @@ impl Segments {
         }
     }
 
-    pub fn set_segment_last_page(&mut self, id: SegmentId, page: u64) {
-        if let Some(s) = self.segments.get_mut(&id) {
-            s.set_last_page(page);
-        }
-    }
-
     pub fn segment_by_id(&self, id: SegmentId) -> Option<&Segment> {
         self.segments.get(&id)
     }
@@ -520,6 +511,20 @@ impl Segments {
         Ok(())
     }
 
+    pub fn recompute_last_pages(&mut self, allocator: &Allocator) -> PERes<()> {
+        for (_, segment) in self.segments.iter_mut() {
+            loop {
+                let mut page = allocator.load_page(segment.last_page)?;
+                let n_page = page.get_next()?;
+                if n_page == 0 {
+                    break;
+                }
+                segment.last_page = n_page;
+            }
+        }
+        Ok(())
+    }
+
     pub fn flush_segments(&mut self, allocator: &Allocator) -> PERes<()> {
         let mut buffer = Vec::<u8>::new();
         for segment in self.segments.values() {
