@@ -3,18 +3,21 @@ use crate::{
     config::TxStrategy,
     error::{IndexChangeError, PERes, TimeoutError},
     id::{index_id_to_segment_id_data, index_id_to_segment_id_meta, IndexId, RecRef, SegmentId},
-    journal::{Journal, JournalId},
-    persy::PersyImpl,
-    snapshots::{release_snapshot, SnapshotEntry, SnapshotId},
-    GenericError, PrepareError,
-};
-use crate::{
     index::{
         config::{IndexTypeInternal, ValueMode},
         keeper::IndexTransactionKeeper,
         tree::nodes::Value,
     },
-    DropSegmentError,
+    journal::{
+        records::{
+            Commit, CreateSegment, DeleteRecord, DropSegment, FreedPage, InsertRecord, Metadata, NewSegmentPage,
+            PrepareCommit, ReadRecord, Rollback, UpdateRecord,
+        },
+        Journal, JournalId,
+    },
+    persy::PersyImpl,
+    snapshots::{release_snapshot, SnapshotEntry, SnapshotId},
+    DropSegmentError, GenericError, PrepareError,
 };
 use std::{
     collections::{hash_map::Entry, BTreeSet, HashMap, HashSet},
@@ -22,78 +25,6 @@ use std::{
     vec::{self, IntoIter},
 };
 
-#[derive(Clone, Default)]
-pub struct NewSegmentPage {
-    pub segment: SegmentId,
-    pub page: u64,
-    pub previous: u64,
-}
-
-#[derive(Clone, Default)]
-pub struct InsertRecord {
-    pub segment: SegmentId,
-    pub recref: RecRef,
-    pub record_page: u64,
-}
-
-#[derive(Clone, Default)]
-pub struct UpdateRecord {
-    pub segment: SegmentId,
-    pub recref: RecRef,
-    pub record_page: u64,
-    pub version: u16,
-}
-
-#[derive(Clone, Default)]
-pub struct ReadRecord {
-    pub segment: SegmentId,
-    pub recref: RecRef,
-    pub version: u16,
-}
-
-#[derive(Clone, Default)]
-pub struct DeleteRecord {
-    pub segment: SegmentId,
-    pub recref: RecRef,
-    pub version: u16,
-}
-
-#[derive(Clone, Default)]
-pub struct CreateSegment {
-    pub name: String,
-    pub segment_id: SegmentId,
-    pub first_page: u64,
-}
-
-#[derive(Clone, Default)]
-pub struct DropSegment {
-    pub name: String,
-    pub segment_id: SegmentId,
-}
-
-#[derive(Clone, PartialEq, Debug, PartialOrd, Ord, Eq, Default)]
-pub struct FreedPage {
-    pub page: u64,
-}
-
-#[derive(Default)]
-pub struct PrepareCommit {}
-
-#[derive(Default)]
-pub struct Commit {}
-
-#[derive(Default)]
-pub struct Cleanup {}
-
-#[derive(Default)]
-pub struct Rollback {}
-
-#[derive(Default)]
-pub struct Metadata {
-    pub strategy: TxStrategy,
-    pub meta_id: Vec<u8>,
-}
-
 pub enum SegmentOperation {
     Create(CreateSegment),
     Drop(DropSegment),
@@ -613,7 +544,6 @@ impl TransactionImpl {
         journal.prepare(&PrepareCommit::new(), &self.id)?;
         allocator.flush_free_list()?;
         self.sync(persy_impl, snapshot_id)?;
-        journal.clear_in_queque()?;
         Ok((self, prepared))
     }
 
@@ -1003,121 +933,15 @@ impl TransactionImpl {
     }
 }
 
-impl DeleteRecord {
-    pub fn new(segment: SegmentId, rec_ref: &RecRef, version: u16) -> DeleteRecord {
-        DeleteRecord {
-            segment,
-            recref: *rec_ref,
-            version,
-        }
-    }
-}
-
-impl UpdateRecord {
-    pub fn new(segment: SegmentId, rec_ref: &RecRef, record: u64, version: u16) -> UpdateRecord {
-        UpdateRecord {
-            segment,
-            recref: *rec_ref,
-            record_page: record,
-            version,
-        }
-    }
-}
-
-impl ReadRecord {
-    pub fn new(segment: SegmentId, recref: &RecRef, version: u16) -> ReadRecord {
-        ReadRecord {
-            segment,
-            recref: *recref,
-            version,
-        }
-    }
-}
-
-impl PrepareCommit {
-    pub fn new() -> PrepareCommit {
-        PrepareCommit {}
-    }
-}
-
-impl Commit {
-    pub fn new() -> Commit {
-        Commit {}
-    }
-}
-
-impl Rollback {
-    pub fn new() -> Rollback {
-        Rollback {}
-    }
-}
-
-impl Cleanup {
-    pub fn new() -> Cleanup {
-        Cleanup {}
-    }
-}
-
-impl InsertRecord {
-    pub fn new(segment: SegmentId, rec_ref: &RecRef, record: u64) -> InsertRecord {
-        InsertRecord {
-            segment,
-            recref: *rec_ref,
-            record_page: record,
-        }
-    }
-}
-
-impl CreateSegment {
-    pub fn new(name: &str, segment_id: SegmentId, first_page: u64) -> CreateSegment {
-        CreateSegment {
-            name: name.into(),
-            segment_id,
-            first_page,
-        }
-    }
-}
-
-impl DropSegment {
-    pub fn new(name: &str, segment_id: SegmentId) -> DropSegment {
-        DropSegment {
-            name: name.into(),
-            segment_id,
-        }
-    }
-}
-
-impl Metadata {
-    pub fn new(strategy: &TxStrategy, meta_id: Vec<u8>) -> Metadata {
-        Metadata {
-            strategy: strategy.clone(),
-            meta_id,
-        }
-    }
-}
-
-impl FreedPage {
-    pub fn new(page: u64) -> FreedPage {
-        FreedPage { page }
-    }
-}
-
-impl NewSegmentPage {
-    pub fn new(segment: SegmentId, page: u64, previous: u64) -> NewSegmentPage {
-        NewSegmentPage {
-            segment,
-            page,
-            previous,
-        }
-    }
-}
-
 #[cfg(test)]
 mod tests {
-    use super::{DeleteRecord, FreedPage, InsertRecord, TransactionImpl, UpdateRecord};
     use crate::{
         id::{RecRef, SegmentId},
-        journal::JournalId,
+        journal::{
+            records::{DeleteRecord, FreedPage, InsertRecord, UpdateRecord},
+            JournalId,
+        },
+        transaction_impl::TransactionImpl,
     };
 
     #[test]
